#!/usr/bin/env python

import numpy as np
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

from logslider import LogSlider, LogSlider0

ω = 1.00
ω0 = 1.00
β = 0.01
fill = 0.5

N = 2000

NN = np.arange(-N, N + 1)

TT = np.linspace(0., 4.0, 2001)


def set_force(f=fill):
    ff = 1.0 * ((TT % 1.0) < f) - 0.5
    fc = 0.5j / (np.pi * NN) * (np.exp(-2j * np.pi * f * NN) - 1.0)
    fc[N] = 0.
    return ff, fc


def ampl(ω0, β, fc):
    NN = np.arange(-N, N + 1)
    return fc / ((ω0**2 - (NN * ω)**2) + 2j * β * NN * ω)


def ift(c):
    c = c[:N + 1]
    c = np.concatenate((c[-1:0:-1].conj(), c))
    NN = np.arange(-N, N + 1)
    return np.sum(c.reshape(-1, 1) * np.exp(2j * np.pi * NN.reshape(-1, 1) * TT.reshape(1, -1)), axis=0)


fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True, gridspec_kw={'hspace': 0.05})

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Oscylator z wymuszeniem prostokątnym")
manager.full_screen_toggle()

ff, fc = set_force(fill)
force, = ax1.plot(TT, ff, lw=1.5)
# _test, = ax1.plot(TT, ift(fc), lw=1.0)

xx = ift(ampl(ω0, β, fc))
oscillations, = ax2.plot(TT, xx, color='C1', lw=1.5)
mx = np.max(np.abs(xx)) * 1.05
ax2.set_ylim(-mx, mx)

ax2.set_xlim(TT[0], TT[-1])
ax2.set_xlabel('$t$ [$2\\pi/\\omega_0$]')

ax1.set_ylabel('Siła wymuszająca')
ax2.set_ylabel("Zachowane układu")

fax = fig.add_axes([0.17, 0.10, 0.75, 0.03])
ωax = fig.add_axes([0.17, 0.07, 0.75, 0.03])
βax = fig.add_axes([0.17, 0.04, 0.75, 0.03])

fig.subplots_adjust(bottom=0.22, top=0.98, left=0.08, right=0.98)

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'black',
}

omega0_slider = LogSlider(ax=ωax, label='$\\omega / \\omega_0$', valmin=-3, valmax=4, valinit=0., valstep=0.01, **sst)
omega0_slider.set_val(np.log10(1./ω0))
beta_slider = LogSlider0(
    ax=βax, label='Tłumienie $\\beta/\\omega_0$', valmin=-3, valmax=2, valstep=0.01, valinit=0, **sst
)
beta_slider.set_val(np.log10(β/ω0))
fill_slider = Slider(ax=fax, label='wypełnienie', valmin=0.0, valmax=1.0, valinit=fill, valstep=0.01, **sst)


def on_slider(val):
    ff, fc = set_force(fill_slider.val)
    # _test.set_ydata(ift(fc))
    ω0 = 1. / omega0_slider.val
    tt = TT * ω0
    β = beta_slider.val * ω0
    xx = ift(ampl(ω0, β, fc))
    force.set_data(tt, ff)
    oscillations.set_data(tt, xx)
    mx = np.max(np.abs(xx)) * 1.05
    ax2.set_ylim(-mx, mx)
    ax2.set_xlim(tt[0], tt[-1])
    fig.canvas.draw_idle()


omega0_slider.on_changed(on_slider)
beta_slider.on_changed(on_slider)
fill_slider.on_changed(on_slider)

plt.show()
